package test;

import javax.annotation.processing.AbstractProcessor;
import javax.annotation.processing.RoundEnvironment;
import javax.lang.model.element.ExecutableElement;
import javax.lang.model.element.TypeElement;
import javax.lang.model.element.VariableElement;
import javax.tools.Diagnostic.Kind;
import javax.tools.FileObject;
import javax.tools.StandardLocation;
import static javax.tools.StandardLocation.CLASS_OUTPUT;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Collection;
import java.util.Collections;
import java.util.Set;

/**
 * @author Kohsuke Kawaguchi
 */
@SuppressWarnings({"Since15"})
public class MyProcessor extends AbstractProcessor {
    @Override
    public Set<String> getSupportedAnnotationTypes() {
        return Collections.singleton(CaptureParameterNames.class.getName());
    }

    public boolean process(Set<? extends TypeElement> annotations, RoundEnvironment env) {
        if(!env.processingOver())   return false;


        System.out.println("*****");
        try {
            for (ExecutableElement method : (Collection<ExecutableElement>) env.getElementsAnnotatedWith(CaptureParameterNames.class)) {
                StringBuilder buf = new StringBuilder();
                for (VariableElement p : method.getParameters()) {
                    if(buf.length()>0)  buf.append(',');
                    buf.append(p.getSimpleName());
                }

                TypeElement type = (TypeElement) method.getEnclosingElement();
                FileObject file = processingEnv.getFiler().createResource(CLASS_OUTPUT, "",
                        type.getQualifiedName().toString().replace('.', '/') + "/" + method.getSimpleName(),
                        method);
                PrintWriter w = new PrintWriter(file.openWriter());
                w.println(buf);
                w.close();
            }
        } catch (IOException e) {
            processingEnv.getMessager().printMessage(Kind.ERROR,
                    "Failed to capture parameter names: "+e.getMessage());
        }
        return false;
    }
}
